#!/bin/python3
# author: Jan Hybs

import os


def divider(name=None, start='# ', end=' #', char='-', len=60):
    """
    Creates a formatted section with given length padded with given chars

    Parameters
    ----------
    name : str or None
        name of the section
    start : str
    end : str
    char: str
    len: int

    Returns
    -------
    str
        formatted string

    """
    fmt = '%s{:%s^%ds}%s' % (start, char, len, end)
    if name:
        return fmt.format(' %s ' % name)
    return fmt.format('')


class TempFile(object):
    """
    A simple class which creates a shell executable file
    using python 'with' syntax

    """

    def __init__(self, path, verbose=False):
        self.path = path
        self.lines = []
        self.verbose = verbose

    def write_shebang(self, interpreter='#!/bin/bash'):
        self.write('%s\n' % interpreter)
        self.write(divider(char='-'))
        self.write(divider('AUTOGENERATED DO NOT EDIT'))
        if self.verbose:
            self.write('set -x')
        self.write(divider(char='-'))
        self.write('')

    def read(self):
        with open(self.path, 'r') as fp:
            return fp.read()

    def write(self, s='', new_line=True):
        self.lines.append(s)
        if new_line:
            self.lines.append('\n')

    def remove(self):
        return os.unlink(self.path)

    def write_section(self, section_name, s='', new_line=True):
        self.write(divider(section_name, char='='))
        self.write(s)
        self.write(divider(section_name, char='-'), new_line=new_line)
        divider()

    def __enter__(self):
        self.lines = []
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        with open(self.path, 'w') as fp:
            for s in self.lines:
                fp.write(s)
        os.chmod(self.path, 0o755)

        return False
